import java.awt.Component;
import java.awt.Cursor;
import java.awt.event.ActionEvent;
import java.util.Locale;
    
import javax.print.PrintService;  
import javax.print.attribute.AttributeSet;
import javax.swing.*;
import javax.swing.text.TextAction;

import jpdf.gui.dialog.PrintDialog;  
import jpdf.gui.dialog.SetupContentDialog;  
import jpdf.gui.dialog.factory.SetupContentPrintDialogFactory;
import jpdf.gui.preview2.PreviewPrintableDialog;
import jpdf.manage.PrintManager;
import jpdf.printer.TextPanePrinter;


/**
 * Class PrintableStylepad adds print capabilities to the Stylepad application.  The 
 * Stylepad is one of the demos that come with the Java SDK.  Three items are added
 * to the File Menu: Page Setup, Print Preview, and Print.  
 *
 * <P>This class was developed to serve as an example of how to use the JPDF to add 
 * printing capabilities to an existing application.
 *
 * <P><HR>
 * This application must be compiled and run using SDK version 1.5.  Version 1.5 
 * is required because of the way in which the Stylepad application was compiled.
 * <HR>
 */
public class PrintableStylepad 
  extends Stylepad 
{
  //-----------------------------------------------------------------------
  // Inner Classes.
  //-----------------------------------------------------------------------

  /**
   * Abstract Print Action.
   */
  protected abstract class AbstractPrintAction 
    extends AbstractAction 
  {
    AbstractPrintAction(String theName) 
    {
      super(theName);
    }

    protected TextPanePrinter createTextComponentPrinter() 
    {
      TextPanePrinter thePrinter = new TextPanePrinter((JTextPane)getEditor(), "Alice In Wonderland");
      
      // Set the maximum empty space at the bottom of a page to 3 inches.
      thePrinter.setMaxEmptySpaceAtPageBottom(3 * 72);
      
      return thePrinter;
    }
  }

  /**
   * Page Setup Action.
   */
  protected class PageSetupAction 
    extends AbstractPrintAction 
  {
    PageSetupAction() 
    {
      super("pageSetup");
    }

    public void actionPerformed(ActionEvent theEvent) 
    {
      SetupContentDialog theDialog = null;
      JFrame             theParentFrame = null;
      Component          theGlassPane = null;

      // Get the frame containing this Printable Stylepad.
      theParentFrame = (JFrame)SwingUtilities.windowForComponent(PrintableStylepad.this);
      theGlassPane = theParentFrame.getGlassPane();

      // Create and display the Page Setup Dialog.
      theParentFrame.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
      theGlassPane.setVisible(true);
      theDialog = 
        SetupContentDialog.createTextComponentContentDialog(theParentFrame, 
                                                            PrintManager.getDefaultPrintAttributes());
      theGlassPane.setVisible(false);
      theParentFrame.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
      theDialog.show(); 
    }
  }

  /**
   * Preview Action.
   */
  protected class PreviewAction 
    extends AbstractPrintAction 
  {
    PreviewAction() 
    {
      super("printPreview");
    }

    public void actionPerformed(ActionEvent theEvent) 
    {
      PreviewPrintableDialog thePreviewDialog = null;
      JFrame                 theParentFrame = null;
      Component              theGlassPane = null;
      TextPanePrinter        thePrinter = createTextComponentPrinter();
      
      // Get the frame containing this Printable Stylepad.
      theParentFrame = (JFrame)SwingUtilities.windowForComponent(PrintableStylepad.this);
      theGlassPane = theParentFrame.getGlassPane();

      // Create and display the Preview Dialog.
      // NOTE: The Dialog Factory creates the Page Setup and Print dialogs that can be displayed
      //       from within the Preview Dialog.
      theGlassPane.setVisible(true);
      theParentFrame.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
      thePreviewDialog = new PreviewPrintableDialog(theParentFrame, thePrinter); 
      thePreviewDialog.setDialogFactory(new SetupContentPrintDialogFactory());                                                      
      theParentFrame.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
      theGlassPane.setVisible(false);
      thePreviewDialog.show(); 
    }
  }

  /**
   * Print Action.
   */
  protected class PrintAction 
    extends AbstractPrintAction 
  {
    PrintAction() 
    {
      super("print");
    }

    public void actionPerformed(ActionEvent theEvent) 
    {
      PrintDialog  theDialog = null;
      JFrame       theParentFrame = null;
      Component    theGlassPane = null;
      PrintService thePrintService = null;
      
      // Get the frame containing this Printable Stylepad.
      theParentFrame = (JFrame)SwingUtilities.windowForComponent(PrintableStylepad.this);
      theGlassPane = theParentFrame.getGlassPane();

      // Create and display the Print Dialog.
      theParentFrame.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
      theGlassPane.setVisible(true);
      theDialog = new PrintDialog(theParentFrame, PrintManager.getDefaultPrintAttributes(), true);
      theGlassPane.setVisible(false);
      theParentFrame.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
      theDialog.show(); 
      
      // Get the Selected Print Service.
      // NOTE: Null is returned if the user did not press the Print button.
      thePrintService = theDialog.getSelectedPrintService();
      if (thePrintService != null) 
      {
        // Print the Text Component.
        PrintManager.printPrintable(createTextComponentPrinter(), theDialog.getPrintInfoHolder());
      }          
    }
  }
    

  //-----------------------------------------------------------------------
  // Constructors.
  //-----------------------------------------------------------------------
  
  public PrintableStylepad()
  {
    super();
  }
  
  
  //-----------------------------------------------------------------------
  // Public methods.
  //-----------------------------------------------------------------------

  /**
   * Fetch the list of actions supported by this editor.  Implemented to return the list
   * of actions supported by the Sytlepad parent class augmented with the locally defined
   * print actions.
   */
  public Action[] getActions() 
  {
    Action[] thePrintActions = new Action[] 
             {new PageSetupAction(), new PreviewAction(), new PrintAction()};
    
    return TextAction.augmentList(super.getActions(), thePrintActions);
  }
}